-------------------------------------------------------------------------------
-- Lorewalkers by Lockslap, US-Bleeding Hollow
-- <Borderline Amazing>, http://ba-guild.com
-------------------------------------------------------------------------------
Lorewalkers = LibStub("AceAddon-3.0"):NewAddon("Lorewalkers", "AceEvent-3.0", "AceConsole-3.0")
if not TomTom then return end

-- libaries
local L = LibStub("AceLocale-3.0"):GetLocale("Lorewalkers", false)
local Astrolabe = DongleStub("Astrolabe-1.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local LSM = LibStub("LibSharedMedia-3.0")

-- font used for waypoint frame info
local LorewalkersFont = CreateFont("LorewalkersFont")
LorewalkersFont:SetJustifyH("LEFT")
LorewalkersFont:SetJustifyV("MIDDLE")

-- waypoints frame
local LorewalkersFrame = CreateFrame("frame", "LorewalkersFrame", UIParent)
local LorewalkersFrameBG = LorewalkersFrame:CreateTexture("LorewalkersFrameBG", "BACKGROUND")
LorewalkersFrame:EnableMouse(true)
LorewalkersFrame:SetResizable(false)
LorewalkersFrame:SetMovable(true)
LorewalkersFrame:SetUserPlaced(true)
LorewalkersFrame:SetPoint("CENTER")
LorewalkersFrameBG:SetAllPoints()

-- font strings used for displaying the criteria and achievement names
LorewalkersFrameString = LorewalkersFrame:CreateFontString("LorewalkersFrameString", "OVERLAY")
LorewalkersFrameString:SetFontObject(LorewalkersFont)
LorewalkersFrameString:SetPoint("TOPLEFT", LorewalkersFrame, 5, -5)

-- add the fonts to LSM
LSM:Register("font", "LW Font", [[Interface\AddOns\Lorewalkers\Fonts\PT_Sans_Narrow.ttf]], LSM.LOCALE_BIT_ruRU + LSM.LOCALE_BIT_western)
LSM:Register("font", "LW Alt Font 1", [[Interface\AddOns\Lorewalkers\Fonts\Action_Man.ttf]])
LSM:Register("font", "LW Alt Font 2", [[Interface\AddOns\Lorewalkers\Fonts\Continuum_Medium.ttf]])

-- setup some globals
Lorewalkers.name 	= GetAddOnMetadata("Lorewalkers", "Title")
Lorewalkers.version = GetAddOnMetadata("Lorewalkers", "Version")

local waypoint
local zones, db, waypoints = {}, {}, {}

local enabledString = ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString = ("|cffff0000%s|r"):format(L["Disabled"])
local repStandings 		= {
	[1]	= L["Hated"],
	[2]	= L["Hostile"],
	[3]	= L["Unfriendly"],
	[4]	= L["Neutral"],
	[5]	= L["Friendly"],
	[6]	= L["Honored"],
	[7]	= L["Revered"],
	[8]	= L["Exalted"],
}

-- options page
local fields = {"Title", "Version", "Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate", "X-Paypal"}
local fieldName = "|cff1784d1%s:|r"
local defaults 		= {
	char = {
		enabled 	= true,
		debug = UnitName("player") == "Lockslap" and true or false,	-- i need debugging by default
		notices = UnitName("player") == "Lockslap" and true or false,	-- ditto for notices
		lockslap = false,
		arrow = true,
		minimap = true,
		worldmap = true,
		
		-- waypoint frame
		frame = {
			enabled = true,
			
			-- frame settings
			locked = false,
			scale = 1.0,
			alpha = 1.0,
			
			-- font settings
			font = "LW Font",
			outline = "OUTLINE",
			size = 15,
			color = {r = 1.0, g = 1.0, b = 1.0}, -- white
		},
		
		-- not used in config frame
		firstLoad = false,	-- for building our waypoint db
		waypoints = {},
	}
}
local options = {
	type = "group",
	args = {
		general = {
			type = "group",
			name = Lorewalkers.name,
			get = function(info) return db[info[#info]] end,
			set = function(info, value) 
				db[info[#info]] = value
				if db.enabled or db.debug then
					if waypoint then
						TomTom:RemoveWaypoint(waypoint)
					end
					Lorewalkers:AddClosestWaypoint()
				else
					if waypoint then TomTom:RemoveWaypoint(waypoint) end
				end
			end,
			args = {
				header = {
					type = "header",
					order = 1,
					name = (L["Version: |cff99ff33%s|r"]):format(Lorewalkers.version),
					width = "full",
				},
				description = {
					type = "description",
					order = 2,
					name = L["A simple addon to aid you with obtaining exalted with The Lorewalkers."],
					width = "full",
				},
				enabled = {
					type = "toggle",
					order = 3,
					name = L["Enable Lorewalkers"],
					desc = L["Toggle the waypoints."],
				},
				debug = {
					type = "toggle",
					order = 5,
					name = L["Debugging"],
					desc = L["This will ignore if you have completed the achievements/criteria and load the waypoints.  I use this for fixing issues."],
				},
				notices = {
					type = "toggle",
					order = 6,
					name = L["Display Notices"],
					desc = L["Notify you of various actions being taken by the addon.  For example, loading a new waypoint."],
				},
				dummy = {
					type = "description",
					order = 99,
					name = "",
					width = "full",
				},
				addTo = {
					type = "group",
					order = 100,
					guiInline = true,
					name = L["Add To"],
					args = {
						arrow = {
							type = "toggle",
							order = 1,
							name = L["Crazy Arrow"],
							desc = L["Add waypoints to TomTom's crazy arrow."],
						},
						minimap = {
							type = "toggle",
							order = 2,
							name = L["Minimap"],
							desc = L["Add waypoints to the minimap."],
						},
						worldmap = {
							type = "toggle",
							order = 3,
							name = L["World Map"],
							desc = L["Add waypoints to the world map."],
						},
					},
				},
			},
		},
		frame = {
			type = "group",
			order = 100,
			name = L["Waypoint Frame"],
			get = function(info) return db.frame[info[#info]] end,
			set = function(info, value)
				db.frame[info[#info]] = value
				Lorewalkers:UpdateFont()
				Lorewalkers:UpdateFrame()
				if db.enabled or db.debug then
					if waypoint then
						TomTom:RemoveWaypoint(waypoint)
					end
					Lorewalkers:AddClosestWaypoint()
				else
					if waypoint then TomTom:RemoveWaypoint(waypoint) end
				end
			end,
			args = {
				enabled = {
					type = "toggle",
					order = 1,
					name = L["Enable Waypoint Frame"],
					desc = L["Toggle the waypoint frame."],
				},
				frameSettings = {
					type = "group",
					order = 99,
					name = L["Frame Settings"],
					disabled = function() return not db.frame.enabled end,
					guiInline = true,
					args = {
						locked = {
							type = "toggle",
							order = 1,
							name = L["Locked"],
							get = function() return db.frame.locked end,
							set = function(_, value)
								db.frame.locked = value
								if db.frame.locked then
									LorewalkersFrame:Hide()
									LorewalkersFrame:SetScript("OnMouseDown", nil)
								else
									LorewalkersFrame:Show()
									LorewalkersFrame:SetScript("OnMouseDown", function() LorewalkersFrame:StartMoving() end)
								end
							end,
						},
						scale = {
							type = "range",
							order = 2,
							name = L["Scale"],
							min = 0.0, max = 3.0, step = 0.1,
						},
						alpha = {
							type = "range",
							order = 3,
							name = L["Transparency"],
							min = 0.0, max = 1.0, step = 0.1,
						},
					},
				},
				fontSettings = {
					type = "group",
					order = 100,
					name = L["Font Settings"],
					guiInline = true,
					disabled = function() return not db.frame.enabled end,
					args = {
						font = {
							type = "select",
							order = 1,
							name = L["Font Face"],
							dialogControl = "LSM30_Font",
							values = AceGUIWidgetLSMlists.font,
						},
						outline = {
							type = "select",
							order = 2,
							name = L["Font Ouline"],
							values = {
								["NORMAL"] = L["Normal"],
								["OUTLINE"] = L["Outline"],
								["THICKOUTLINE"] = L["Thick"],
							},
						},
						size = {
							type = "range",
							order = 3,
							name = L["Font Size"],
							min = 9, max = 21, step = 1,
						},
						color = {
							type = "color",
							order = 4,
							name = L["Font Color"],
							get = function() return db.frame.color.r, db.frame.color.g, db.frame.color.b end,
							set = function(_, r, g, b)
								db.frame.color = {
									r = r,
									g = g,
									b = b,
								}
								Lorewalkers:UpdateFont()
								Lorewalkers:UpdateFrame()
								if db.enabled or db.debug then
									if waypoint then
										TomTom:RemoveWaypoint(waypoint)
									end
									Lorewalkers:AddClosestWaypoint()
								else
									if waypoint then TomTom:RemoveWaypoint(waypoint) end
								end
							end,
						},
					},
				},
			},
		},
	},
}

local function InArray(needle, haystack)
	if #haystack == 0 then return false end
	for i = 1, #haystack do
		if haystack[i] == needle then
			return true
		end
	end
	return false
end

local function RepStatus()
	-- id 1345 are the lorewalkers
	local _, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(1345)
	barValue = barValue - barMin
	barMax = barMax - barMin
	return barValue, barMax, repStandings[standing], math.floor(barValue / barMax * 100)
end

local function GetCriteriaCoordinates(criteria)
	for zone, waypoints in pairs(Lorewalkers.waypoints) do
		for name, info in pairs(waypoints) do
			if criteria == name then
				return zone, info.x, info.y
			end
		end
	end
	return false
end

function Lorewalkers:AddAllWaypoints()
	if GetCurrentMapContinent() ~= 6 or not db.waypoints[GetCurrentMapAreaID()] then return end
	local map, opts = GetCurrentMapAreaID(), {
		title = "",
		minimap = false,
		crazy = true,
		world = true,
		persistent = false,
		cleardistance = 0,
		arrivaldistance = 0,
		callbacks = {},
	}
	for criteria, point in pairs(db.waypoints[map]) do
		_G["ChatFrame3"]:AddMessage(("|cffffff00%s|r |cff00ff00%s|r %d %d"):format(point.achieveName, point.name, map, criteria))
		opts.title = ("%s\n%s"):format(point.name, point.achieveName)
		waypoints[#waypoints + 1] = TomTom:AddMFWaypoint(map, 0, point.x / 100, point.y / 100, opts)
	end
end

function Lorewalkers:ClearAllWaypoints()
	if not waypoints or #waypoints == 0 then return end
	for i = 1, #waypoints do
		TomTom:RemoveWaypoint(waypoints[i])
	end
end

function Lorewalkers:UpdateCriteria()
	for mapID, waypoints in pairs(db.waypoints) do
		for cID, point in pairs(waypoints) do
			local _, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(point.achieveID, point.num)
			db.waypoints[mapID][cID].completed = completed
		end
	end
end

function Lorewalkers:HandleChatCommand(args)
	local command, value = self:GetArgs(args, 2)
	local iconString = "\124T%s:12\124t"
	local completedString = "%s%s %s |cff1784d1%d/%d/%d|r"
	local notCompletedString = "%s%s |cffff0000%s|r"
	local criteriaCompleted = "    %d. %s: |cffffff00%s|r%s |cff00ff00%s|r"
	local criteriaNotCompleted = "    %d. %s: |cffffff00%s|r%s |cffff0000%s|r"
	local achievementPattern = "\124c%x+\124Hachievement:(%d-):.-\124h%[.-%]\124h\124r"
	
	if command == "toggle" then
		db.enabled = not db.enabled
		self:Print((L["Waypoints have been %s"]):format(db.enabled == true and enabledString or disabledString))
		if not db.enabled and waypoint then
			LorewalkersFrame:Hide()
			TomTom:RemoveWaypoint(waypoint)
		else
			self:UpdateCriteria()
			self:AddClosestWaypoint()
		end
		
	elseif command == "config" then
		InterfaceOptionsFrame_OpenToCategory(self.frames.general)
		
	elseif command == "rep" or command == "reputation" then
		local exaltedString = "%s |cff1784d1%d/%d %s (%d%%)|r %s |cff1784d1%s|r."
		local value, max, standing, percent = RepStatus()
		self:Print(exaltedString:format(L["You are"], value, max, standing, percent, L["with"], L["The Lorewalkers"]))
		
	elseif command == "criterias" or command == "criteria" or (value and value:match(achievementPattern)) then
		local id = 0
		if value:match(achievementPattern) then
			id = value:match(achievementPattern)
		elseif value:find(" ") then
			-- they provided a string, it must be in quotes
			if not Lorewalkers.achievements[value] then
				self:Print((L["Achievement |cffff0000\"%s\"|r doesn't exist."]):format(value))
				return
			else
				id = Lorewalkers.achievements[value]
			end
		else
			id = tonumber(value)
		end
		local link = GetAchievementLink(id)
		local _, _, _, completed, month, day, year, _, _, icon, _, _ = GetAchievementInfo(id)
		self:Print(completed == true and completedString:format(iconString:format(icon), link, L["Completed on"], month, day, year) or notCompletedString:format(iconString:format(icon), link, L["Incomplete"]))
		for i = 1, GetAchievementNumCriteria(id) do
			local description, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(id, i)
			local zone, x, y = GetCriteriaCoordinates(description)
			if zone then
				print(completed == true and criteriaCompleted:format(i, description, zone, ("(%.2f, %.2f)"):format(x, y), L["Completed"]) or criteriaNotCompleted:format(i, description, zone, ("(%.2f, %.2f)"):format(x, y), L["Incomplete"]))
			else
				print(completed == true and criteriaCompleted:format(i, description, "", "", L["Completed"]) or criteriaNotCompleted:format(i, description, "", "", L["Incomplete"]))
			end
		end
		
	elseif command == "achieve" or command == "achievements" then
		for _, id in pairs(self.achievements) do
			local link = GetAchievementLink(id)
			local _, _, _, completed, month, day, year, _, _, icon, _, _ = GetAchievementInfo(id)
			self:Print(completed == true and completedString:format(iconString:format(icon), link, L["Completed on"], month, day, year) or notCompletedString:format(iconString:format(icon), link))
		end
		
	elseif command == "clear" then
		if waypoint then TomTom:RemoveWaypoint(waypoint) end
		self:ClearAllWaypoints()	-- for "lockslap" mode
	
	elseif command == "reload" then
		-- can't believe i havent added this until now
		if waypoint then TomTom:RemoveWaypoint(waypoint) end
		LorewalkersFrame:Hide()
		self:ClearAllWaypoints()	-- for "lockslap" mode
		self:AddClosestWaypoint()
	
	elseif command == "debug" then
		db.debug = not db.debug
		self:Print((L["Debugging has been %s"]):format(db.debug == true and enabledString or disabledString))
		if not db.debug and waypoint then
			LorewalkersFrame:Hide()
			TomTom:RemoveWaypoint(waypoint)
		else
			self:UpdateCriteria()
			self:AddClosestWaypoint()
		end
	
	-- my own command to make taking screenshots easier
	-- this simply ignores if waypoints are completed and loads them all
	elseif command == "lockslap" then
		db.lockslap = not db.lockslap
		self:Print((L["Lockslap mode %s"]):format(db.lockslap == true and enabledString or disabledString))
		if db.lockslap then
			db.enabled = false
			db.debug = false
			if GetCurrentMapContinent() == 6 then
				self:AddAllWaypoints()
			end
		else
			self:ClearAllWaypoints()
		end
		
	-- more debugging
	elseif command == "waypoints" then
		local map = GetCurrentMapAreaID()
		if not GetCurrentMapContinent() ~= 6 then return end
		for criter, point in pairs(db.waypoints[map]) do
			_G["ChatFrame3"]:AddMessage(("|cffffff00%s|r |cff00ff00%s|r %d %d"):format(point.achieveName, point.name, map, criteria))
		end
		
	elseif not command or command == "" or command == "help" or command == "?" then
		self:Print((L["Lorewalkers version |cff1784d1%.2f|r by |cff00ff00Lockslap|r"]):format(self.version))
		self:Print(L["Chat Command:  |cff1784d1/lorewalkers|r or |cff1784d1/lw|r"])
		print(L["    |cff1784d1config|r - Open Configuration Window"])
		print(L["    |cff1784d1clear|r - Clear Waypoints"])
		print(L["    |cff1784d1help|r or |cff1784d1?|r - Print This Help Again"])
		print(L["    |cff1784d1toggle|r - Toggle the Waypoints"])
		print(L["    |cff1784d1rep|r or |cff1784d1reputation|r - Your Reputation with the Lorewalkers"])
		print(L["    |cff1784d1criteria|r or |cff1784d1criterias|r |cff1784d1<id or name>|r - Display Achievement Criteria"])
		print(L["    |cff1784d1achievements|r or |cff1784d1achieve|r - Lorewalkers Achievement Progress"])
		
	end
end

function Lorewalkers:GetClosestWaypoint()
	local distance, point, criteriaID
	local mapID = GetCurrentMapAreaID()
	local map, level, x, y = Astrolabe:GetCurrentPlayerPosition()
	if not db.waypoints[mapID] then return false end
	for criteria, points in pairs(db.waypoints[mapID]) do
		local wayDistance = Astrolabe:ComputeDistance(map, level, x, y, mapID, 0, points.x / 100, points.y / 100)
		if (db.debug and wayDistance > 50) or not points.completed then
			if not distance or wayDistance < distance then
				distance = wayDistance
				point = points
				criteriaID = criteria
			end
		end
	end
	-- we should have our nearest waypoint
	if not distance or not point then
		return false
	else
		return mapID, criteriaID, point, distance
	end
end

function Lorewalkers:AddClosestWaypoint()
	local zone = GetZoneText()
	if db.debug and GetCurrentMapContinent() == 6 and not db.waypoints[GetCurrentMapAreaID()] then
		self:Print((L["No waypoints found for |cff1784d1%s|r."]):format(zone))
	elseif GetCurrentMapContinent() == 6 and (db.debug or db.enabled) then
		local mapID, criteriaID, waypoint, distance = self:GetClosestWaypoint()
		if not mapID or not criteriaID or not waypoint or not distance then
			if db.notices then
				self:Print((L["You have completed all waypoints for |cff1784d1%s|r"]):format(zone))
			end
		else
			if db.debug or db.notices then
				self:Print((L["Loaded waypoint for |cff1784d1%s|r, |cff1784d1%.2f|r yards away."]):format(waypoint.name, distance))
			end
		
			-- thanks to Cladhaire, the author of TomTom, for pointing me in the right direction
			local opts = {
				title = ("%s\n%s"):format(waypoint.name, waypoint.achieveName),
				persistent = false,
				minimap = db.minimap,
				world = db.worldmap,
				crazy = db.arrow,
				cleardistance = 0,
				arrivaldistance = 0,
				callbacks = {},
			}
			opts.callbacks = TomTom:DefaultCallbacks(opts)
			
			-- to change the waypoint
			opts.callbacks.distance[15] = function(event, uid, range, distance, lastDistance)
				if lastDistance > distance then
					Lorewalkers:UpdateCriteria()
					TomTom:RemoveWaypoint(uid)
					waypoint = nil
					Lorewalkers:AddClosestWaypoint()
					if db.frame.enabled and db.frame.locked then
						LorewalkersFrame:Hide()
					end
				end
			end
			opts.callbacks.distance[100] = function(event, uid, range, distance, lastDistance)
				if ((not lastDistance and distance <= 100) or (lastDistance > distance)) and db.frame.enabled then
					LorewalkersFrame:Show()
				elseif distance > 100 and db.frame.enabled then
					LorewalkersFrame:Hide()
				end
			end
			waypoint = TomTom:AddMFWaypoint(mapID, 0, waypoint.x / 100, waypoint.y / 100, opts)
			
			if db.frame.enabled then
				self:UpdateFont()
				self:UpdateFrame()
				self:SetScreenshot(mapID, criteriaID)
				
				if db.frame.locked then
					LorewalkersFrame:Hide()
				end
			end
		end
	end
end

function Lorewalkers:HandleEvents(event, ...)
	if waypoint then
		TomTom:RemoveWaypoint(waypoint)
		waypoint = nil
	end
	
	if not db.lockslap then
		if GetCurrentMapContinent() == 6 and (db.debug or db.enabled) then
			self:AddClosestWaypoint()
		end
	else
		self:ClearAllWaypoints()
		self:AddAllWaypoints()
	end
end
Lorewalkers:RegisterEvent("ZONE_CHANGED_NEW_AREA", "HandleEvents")

function Lorewalkers:OnInitialize()
	-- load the config
	self.db = LibStub("AceDB-3.0"):New("LorewalkersDB", defaults)
	db = self.db.char
	
	-- setup tomtom
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.arrival = 5
	
	-- this will be done the first time the addon is loaded
	if not db.firstLoad then
		for mapID, waypoints in pairs(self.waypoints) do
			for cID, point in pairs(waypoints) do
				local _, achieveName, _, _, _, _, _, _, _, _, _, _, _ = GetAchievementInfo(point.achieveID)
				for i = 1, GetAchievementNumCriteria(point.achieveID) do
					local description, _, completed, _, _, _, _, _, _, criteriaID = GetAchievementCriteriaInfo(point.achieveID, i)
					if cID == criteriaID then
						if not db.waypoints[mapID] then db.waypoints[mapID] = {} end
						if not db.waypoints[mapID][cID] then db.waypoints[mapID][cID] = {} end
						db.waypoints[mapID][cID] = {
							achieveID = point.achieveID,
							achieveName = achieveName,
							name = description,
							num = i,
							x = point.x,
							y = point.y,
							completed = completed,
						}
					end
				end
			end
		end
		db.firstLoad = true
	else
		self:UpdateCriteria()
	end
	
	-- add the about group
	options.args.about = {
		type = "group",
		order = 101,
		name = L["About"],
		args = {},
	}
	local i = 1
	for _, field in pairs(fields) do
		local val = GetAddOnMetadata(self.name, field)
		if val then
			options.args.about.args[field] = {
				type = "description",
				order = i,
				width = "normal",
				name = fieldName:format(field:gsub("X%-", "")),
			}
			i = i + 1
			options.args.about.args[("%s_value"):format(field)] = {
				type = "description",
				order = i,
				width = "double",
				name = val,
			}
			i = i + 1
		end
	end

	-- setup config window
	AceConfigRegistry:RegisterOptionsTable(self.name, options)
	self.frames = {}
	self.frames.general = AceConfigDialog:AddToBlizOptions(self.name, nil, nil, "general")
	self.frames.frame = AceConfigDialog:AddToBlizOptions(self.name, L["Waypoint Frame"], self.name, "frame")
	self.frames.about = AceConfigDialog:AddToBlizOptions(self.name, L["About"], self.name, "about")
	
	-- setup the waypoint frame
	if db.frame.locked then
		LorewalkersFrame:Hide()
	else
		LorewalkersFrame:Show()
	end
	self:UpdateFont()
	self:UpdateFrame()
	LorewalkersFrameBG:SetTexture([[Interface\AddOns\Lorewalkers\Waypoints\Default.tga]])
end

function Lorewalkers:OnEnable()
	self:RegisterChatCommand("lorewalkers", "HandleChatCommand")
	self:RegisterChatCommand("lw", "HandleChatCommand")
	
	-- pandaria's continent id is 6
	if GetCurrentMapContinent() == 6 and (db.debug or db.enabled) then
		self:AddClosestWaypoint()
	elseif GetCurrentMapContinent() == 6 and db.lockslap then
		self:AddAllWaypoints()
	end
end

function Lorewalkers:OnDisable()
	self:UnregisterChatCommand("lorewalkers")
	self:UnregisterChatCommand("lw")
	
	if waypoint then
		TomTom:RemoveWaypoint(waypoint)
		waypoint = nil
	end
end

-------------------------------------------------------------------------------
-- Waypoint Screenshot Frame
-------------------------------------------------------------------------------
function Lorewalkers:SetScreenshot(zone, id)
	local image = "Interface\\AddOns\\Lorewalkers\\Waypoints\\%d\\%d.tga"
	LorewalkersFrameBG:SetTexture(image:format(zone, id))
	
	-- set the waypoint frame string
	LorewalkersFrameString:SetText(db.waypoints[zone][id].name .. "\n" .. db.waypoints[zone][id].achieveName)
end

function Lorewalkers:UpdateFrame()
	LorewalkersFrame:SetHeight(256 * db.frame.scale)
	LorewalkersFrame:SetWidth(256 * db.frame.scale)
	LorewalkersFrame:SetAlpha(db.frame.alpha)
end

function Lorewalkers:UpdateFont()
	LorewalkersFont:SetFont(LSM:Fetch("font", db.frame.font), db.frame.size, db.frame.outline)
	LorewalkersFont:SetTextColor(db.frame.color.r, db.frame.color.g, db.frame.color.b, 1.0)
end

-- move the waypoint frame
LorewalkersFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
LorewalkersFrame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
LorewalkersFrame:SetScript("OnLoad", function(self) self:SetUserPlaced(1) end)